==> percona-server-mongodb_4.0.27.orig.tar.gz: percona-server-mongodb-4.0.27-22/src/mongo/db/storage/kv/kv_prefix.cpp <==

/**
 *    Copyright (C) 2018-present MongoDB, Inc.
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the Server Side Public License, version 1,
 *    as published by MongoDB, Inc.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    Server Side Public License for more details.
 *
 *    You should have received a copy of the Server Side Public License
 *    along with this program. If not, see
 *    <http://www.mongodb.com/licensing/server-side-public-license>.
 *
 *    As a special exception, the copyright holders give permission to link the
 *    code of portions of this program with the OpenSSL library under certain
 *    conditions as described in each individual source file and distribute
 *    linked combinations including the program with the OpenSSL library. You
 *    must comply with the Server Side Public License in all respects for
 *    all of the code used other than as permitted herein. If you modify file(s)
 *    with this exception, you may extend this exception to your version of the
 *    file(s), but you are not obligated to do so. If you do not wish to do so,
 *    delete this exception statement from your version. If you delete this
 *    exception statement from all source files in the program, then also delete
 *    it in the license file.
 */

#include "mongo/db/storage/kv/kv_prefix.h"

namespace mongo {
int64_t KVPrefix::_nextValue = 0;
stdx::mutex KVPrefix::_nextValueMutex;
const KVPrefix KVPrefix::kNotPrefixed = KVPrefix(-1);

std::string KVPrefix::toString() const {
    StackStringBuilder ss;
    ss << "KVPrefix(" << _value << ")";
    return ss.str();
}


/* static */ KVPrefix KVPrefix::fromBSONElement(const BSONElement value) {
    if (value.eoo()) {
        return kNotPrefixed;
    }

    return KVPrefix(static_cast<int64_t>(value.Long()));
}

/* static */ void KVPrefix::setLargestPrefix(KVPrefix largestPrefix) {
    if (!storageGlobalParams.groupCollections) {
        return;
    }

    stdx::lock_guard<stdx::mutex> lk(_nextValueMutex);
    _nextValue = largestPrefix._value + 1;
}

/* static */ KVPrefix KVPrefix::getNextPrefix(const NamespaceString& ns) {
    if (!storageGlobalParams.groupCollections || ns.isOplog()) {
        return kNotPrefixed;
    }

    return generateNextPrefix();
}

/* static */ KVPrefix KVPrefix::generateNextPrefix() {
    stdx::lock_guard<stdx::mutex> lk(_nextValueMutex);
    return KVPrefix(_nextValue++);
}
}

==> percona-server-mongodb_4.4.16.orig.tar.gz: percona-server-mongodb-4.4.16-16/src/mongo/db/storage/kv/kv_prefix.cpp <==
/**
 *    Copyright (C) 2018-present MongoDB, Inc.
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the Server Side Public License, version 1,
 *    as published by MongoDB, Inc.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    Server Side Public License for more details.
 *
 *    You should have received a copy of the Server Side Public License
 *    along with this program. If not, see
 *    <http://www.mongodb.com/licensing/server-side-public-license>.
 *
 *    As a special exception, the copyright holders give permission to link the
 *    code of portions of this program with the OpenSSL library under certain
 *    conditions as described in each individual source file and distribute
 *    linked combinations including the program with the OpenSSL library. You
 *    must comply with the Server Side Public License in all respects for
 *    all of the code used other than as permitted herein. If you modify file(s)
 *    with this exception, you may extend this exception to your version of the
 *    file(s), but you are not obligated to do so. If you do not wish to do so,
 *    delete this exception statement from your version. If you delete this
 *    exception statement from all source files in the program, then also delete
 *    it in the license file.
 */

#include "mongo/db/storage/kv/kv_prefix.h"

namespace mongo {
int64_t KVPrefix::_nextValue = 0;
Mutex KVPrefix::_nextValueMutex = MONGO_MAKE_LATCH("KVPrefix::_nextValueMutex");
const KVPrefix KVPrefix::kNotPrefixed = KVPrefix(-1);

std::string KVPrefix::toString() const {
    StackStringBuilder ss;
    ss << "KVPrefix(" << _value << ")";
    return ss.str();
}


/* static */ KVPrefix KVPrefix::fromBSONElement(const BSONElement value) {
    if (value.eoo()) {
        return kNotPrefixed;
    }

    return KVPrefix(static_cast<int64_t>(value.Long()));
}

/* static */ void KVPrefix::setLargestPrefix(KVPrefix largestPrefix) {
    if (!storageGlobalParams.groupCollections) {
        return;
    }

    stdx::lock_guard<Latch> lk(_nextValueMutex);
    _nextValue = largestPrefix._value + 1;
}

/* static */ KVPrefix KVPrefix::getNextPrefix(const NamespaceString& ns) {
    if (!storageGlobalParams.groupCollections || ns.isOplog()) {
        return kNotPrefixed;
    }

    return generateNextPrefix();
}

/* static */ KVPrefix KVPrefix::generateNextPrefix() {
    stdx::lock_guard<Latch> lk(_nextValueMutex);
    return KVPrefix(_nextValue++);
}
}  // namespace mongo
